@@ -53,6 +53,8 @@ struct UICStruct {
 	gboolean dial;
 	gboolean gsm;
 	gboolean transformPlus;
+	gboolean getCalls;
+	gboolean getSms;
 	gulong sigId;
 };
 typedef struct UICStruct UIComponents;
@@ -77,18 +79,22 @@ static void sendSMS (gchar *number, gchar *text);
 #define APPLICATION_DBUS_SERVICE "phonelink"
 
 #define GCONF_TRANSFORM_PLUS_KEY "/apps/phonelink/TransformPlusIntoDoubleZero"
+#define GCONF_NOTIFY_CALL_KEY "/apps/phonelink/NotifyMeForIncomingCall"
+#define GCONF_NOTIFY_SMS_KEY "/apps/phonelink/NotifyMeForSMSReception"
 
 static void  quit_event( UIComponents *uic, guint callback_action, GtkWidget *menu_item	) {
 	GConfClient *gconf_client = NULL;
 
 	gconf_client = gconf_client_get_default ();
 	gconf_client_set_bool(gconf_client, GCONF_TRANSFORM_PLUS_KEY, uic->transformPlus, NULL);
+	gconf_client_set_bool(gconf_client, GCONF_NOTIFY_CALL_KEY, uic->getCalls, NULL);
+	gconf_client_set_bool(gconf_client, GCONF_NOTIFY_SMS_KEY, uic->getSms, NULL);
 	gconf_client_suggest_sync (gconf_client, NULL);
 	g_object_unref (gconf_client);
 
         gn_lib_phoneprofile_free(&state);
         gn_lib_library_free();
-			
+
 	gtk_main_quit ();
 	osso_deinitialize(uic->osso_context);
 	g_free(uic);
@@ -145,12 +151,9 @@ void oneResultSignal_cb(
 				gint retCode, 
 				UIComponents *uic) {
 	GtkTreeIter iter;
-	gchar *text;
 					
 	if (retCode == CI_NOT_FOUND) {
-		text = g_strdup_printf(_("ContactInfos found nothing for %s"), (gchar *) gtk_entry_get_text(GTK_ENTRY(uic->searchEntry)));
-		hildon_banner_show_information(GTK_WIDGET(gtk_widget_get_toplevel(uic->searchEntry)), GTK_STOCK_DIALOG_INFO, text);
-		g_free(text);
+		hildon_banner_show_informationf(GTK_WIDGET(gtk_widget_get_toplevel(uic->searchEntry)), GTK_STOCK_DIALOG_INFO, _("ContactInfos found nothing for %s"), (gchar *) gtk_entry_get_text(GTK_ENTRY(uic->searchEntry)));
 	}
 					
 	if (retCode >= CI_NO_ERROR) {
@@ -183,7 +186,7 @@ void on_menu_quit(GtkMenuItem * menuitem, UIComponents *uic) {
 }
 
 void on_menu_about(GtkMenuItem * menuitem, UIComponents *uic) {
-	gtk_show_about_dialog(GTK_WIDGET(gtk_widget_get_toplevel(menuitem)),
+	gtk_show_about_dialog(GTK_WINDOW(gtk_widget_get_toplevel(GTK_WIDGET(menuitem))),
 		"version", VERSION,
 		"website", "https://garage.maemo.org/projects/phonelink/",
 		"website-label", _("Garage Page @ Maemo"),
@@ -195,6 +198,98 @@ void on_menu_about(GtkMenuItem * menuitem, UIComponents *uic) {
 void on_menu_transform(GtkCheckMenuItem *checkMenuitem, UIComponents *uic) {
 	uic->transformPlus = !(uic->transformPlus);
 }
+/*
+void incoming_call(int lowlevel_id, char *number) {
+*/
+	/* sometimes the callback function gets called twice for the same call */
+/*	if (call_detected == lowlevel_id) return;
+	
+	printf("%s\n", number);
+	call_detected = lowlevel_id;
+	quit = 1;
+}
+
+gn_error hangup(int lowlevel_id) {
+	gn_call_info	callinfo;
+	gn_data		data;
+	
+	memset(&callinfo, 0, sizeof(callinfo));
+	callinfo.call_id = lowlevel_id;
+
+	gn_data_clear(&data);
+	data.call_info = &callinfo;
+
+	return gn_sm_functions(GN_OP_CancelCall, &data, state);
+}
+
+gn_error answer(int lowlevel_id) {
+	gn_call_info	callinfo;
+	gn_data		data;
+	
+	memset(&callinfo, 0, sizeof(callinfo));
+	callinfo.call_id = lowlevel_id;
+
+	gn_data_clear(&data);
+	data.call_info = &callinfo;
+
+	return gn_sm_functions(GN_OP_AnswerCall, &data, state);
+}
+*/
+
+void notify_sms(gn_sms *message, struct gn_statemachine *state, UIComponents *uic) {
+	/* in this callback function you can't use those libgnokii functions that send a packet */
+	
+	hildon_banner_show_informationf(GTK_WIDGET(gtk_widget_get_toplevel(uic->searchEntry)), GTK_STOCK_DIALOG_INFO, _("SMS received from %s :\n%s"), message->remote, message->user_data[0].u.text);
+}
+
+void notify_call(gn_call_status call_status, gn_call_info *call_info, struct gn_statemachine *state, UIComponents *uic) {
+	/* in this callback function you can't use those libgnokii functions that send a packet */
+	
+	if (call_status != GN_CALL_Incoming) return;
+	
+	hildon_banner_show_informationf(GTK_WIDGET(gtk_widget_get_toplevel(uic->searchEntry)), GTK_STOCK_DIALOG_INFO, _("A call from %s"), call_info->number);
+	/*
+	incoming_call(call_info->call_id, call_info->number);
+	*/
+}
+
+void on_menu_getCalls(GtkCheckMenuItem *checkMenuitem, UIComponents *uic) {
+	gn_data		data;
+	gn_error	error = GN_ERR_NONE;
+	
+	uic->getCalls = !(uic->getCalls);
+	if (uic->getCalls) {
+		/* Set up so that we get notified of incoming calls, if supported by the driver */
+		gn_data_clear(&data);
+		data.call_notification = notify_call;
+		data.callback_data = uic;
+		error = gn_sm_functions(GN_OP_SetCallNotification, &data, state);
+		if (error != GN_ERR_NONE) {
+			hildon_banner_show_informationf(GTK_WIDGET(gtk_widget_get_toplevel(GTK_WIDGET(checkMenuitem))), GTK_STOCK_DIALOG_INFO, _("This driver doesn't support call signaling.\n(%s)"), gn_error_print(error));
+			uic->getCalls = !(uic->getCalls);
+			gtk_check_menu_item_set_active(checkMenuitem, uic->getCalls);
+		}
+	}
+}
+
+void on_menu_getSms(GtkCheckMenuItem *checkMenuitem, UIComponents *uic) {
+	gn_data		data;
+	gn_error	error = GN_ERR_NONE;
+	
+	uic->getSms = !(uic->getSms);
+	if (uic->getSms) {
+		/* Set up so that we get notified of incoming sms, if supported by the driver */
+		gn_data_clear(&data);
+		data.on_sms = notify_sms;
+		data.callback_data = uic;
+		error = gn_sm_functions(GN_OP_OnSMS, &data, state);
+		if (error != GN_ERR_NONE) {
+			hildon_banner_show_informationf(GTK_WIDGET(gtk_widget_get_toplevel(GTK_WIDGET(checkMenuitem))), GTK_STOCK_DIALOG_INFO, _("This driver doesn't support sms signaling.\n(%s)"), gn_error_print(error));
+			uic->getSms = !(uic->getSms);
+			gtk_check_menu_item_set_active(checkMenuitem, uic->getSms);
+		}
+	}
+}
 
 GtkMenu *create_menus(UIComponents *uic) {
 	GtkWidget *menuBar;
@@ -225,6 +320,16 @@ GtkMenu *create_menus(UIComponents *uic) {
 	gtk_check_menu_item_set_active (GTK_CHECK_MENU_ITEM(menuItem), uic->transformPlus);
 	gtk_menu_append(menuGroup, menuItem);
 	
+	menuItem = gtk_check_menu_item_new_with_label (_("Notify me on incoming call"));
+	g_signal_connect (G_OBJECT (menuItem), "toggled", G_CALLBACK (on_menu_getCalls), (gpointer) uic);
+	gtk_check_menu_item_set_active (GTK_CHECK_MENU_ITEM(menuItem), uic->getCalls);
+	gtk_menu_append(menuGroup, menuItem);
+	
+	menuItem = gtk_check_menu_item_new_with_label (_("Notify me on SMS reception"));
+	g_signal_connect (G_OBJECT (menuItem), "toggled", G_CALLBACK (on_menu_getSms), (gpointer) uic);
+	gtk_check_menu_item_set_active (GTK_CHECK_MENU_ITEM(menuItem), uic->getSms);
+	gtk_menu_append(menuGroup, menuItem);
+	
 	menuGroup = gtk_menu_new();
 	menuItem = gtk_menu_item_new_with_label (_("Help"));
 	gtk_menu_item_set_submenu(GTK_MENU_ITEM(menuItem), menuGroup);
@@ -508,9 +613,11 @@ static void dial (UIComponents * uic) {
 	}
 }
 
+/*
 static void printData(gpointer data, gpointer data2) {
 	g_print("%s; ", (gchar *) data);
 }
+*/
 
 static gchar * getSMSText(UIComponents * uic) {
 	
@@ -858,6 +965,8 @@ int main( int argc, char *argv[] ) {
 	/* Read GConf Info */
 	gconf_client = gconf_client_get_default ();
 	uic->transformPlus = gconf_client_get_bool(gconf_client, GCONF_TRANSFORM_PLUS_KEY, NULL);
+	uic->getCalls= gconf_client_get_bool(gconf_client, GCONF_NOTIFY_CALL_KEY, NULL);
+	uic->getSms= gconf_client_get_bool(gconf_client, GCONF_NOTIFY_SMS_KEY, NULL);
 
 	hildon_window_set_menu (appview, create_menus (uic));
 
@@ -868,7 +977,7 @@ int main( int argc, char *argv[] ) {
 		gnokiiCfg = getGnokiiCfg(gconf_client, NULL);
 		gnokiiFile = TRUE;
 		state = g_new0(struct gn_statemachine,1);
-		if ((err = gn_cfg_memory_read ((char **)gnokiiCfg)) != GN_ERR_NONE) {
+		if ((err = gn_cfg_memory_read ((const char **)gnokiiCfg)) != GN_ERR_NONE) {
 			hildon_banner_show_informationf(GTK_WIDGET(appview), GTK_STOCK_DIALOG_ERROR, _("Error reading gnokii configuration %s\n%s\n%s\n%s\n%s"), gn_error_print(err), gnokiiCfg[0], gnokiiCfg[1], gnokiiCfg[2], gnokiiCfg[3]);
 			gnokiiFile = FALSE;
 		} else if ((err = gn_cfg_phone_load(NULL, state)) != GN_ERR_NONE) {
